/* declust removes aftershocks and other "dependent" events from
and earthquake catalog, based on Reasenberg (1985), Savage &
DePolo (1993), and some of my own modifications.

Uses the U. of W. "A" card format.
User is responsible for having consistent magnitude type
in the magnitude column. -l and -b options specify ML and mb,
program converts magnitudes to Mw.
If starting year is 18xx, use -c option.
Set Omori parameters with -o option:
	default = "provisional" generic Utah model (Arabasz and Hill, 1994)
	1 = generic California model (Reasenberg and Jones, 1989)
	2 = tau determined as in Reasenberg (1985)

R. LaForge 4/95
*/
 
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#define MAXCLUS	1000

void abdays();

int modays[] = {31,28,31,30,31,30,31,31,30,31,30,31};
int cudays[] = {0,31,59,90,120,151,181,212,243,273,304,334,365};
int culdays[] = {0,31,60,91,121,152,182,213,244,274,305,335,366};
int yrx,monx,dayx,startyear=1800;
float avtim;

main(argc,argv)
	int argc;
	char **argv;
	{
     char line1[90],line2[90],c1[3],c2[3],c3[3],c4[3],c5[3];
	char c6[4],c7[4],c8[4],c9[4],c10[4],c11[4];
	char schar[8],magtyp[]="w";
	float mag1,mag2,mwf2,t1,mo,maindist,r2,xp,pw;
	float day,hr,min,tauterm;
	float latdeg,latmin,latpt,londeg,lonmin,lonpt;
	float mainlat,lat2,mainlon,lon2,pi,rad,cosavlat,dlat,dlon,ddist;
	float tau,taumin,taumax0,taumax,km=111.2;
	float sd,fbars,maxdep,mainmag,Q,P,zdist,zmin;
	float delt,delt2,delm,fppart,avlat,avlon,wt,newmag;
	float sumlat,sumlon,sumwt,sumtim;
	float cluslat[MAXCLUS],cluslon[MAXCLUS],clusmag[MAXCLUS];
	float clustim[MAXCLUS],maintime,time1,time2,minmag;
	float a,b,c,p,term1,term2;
	int cudays[] = {0,31,59,90,120,151,181,212,243,273,304,334};
	int cflag=1,startyear,nclus=0,year1,year2,lpdays,yrmon;
	int yr1,yr2,mon,cswitch=0,nind=0,nclusters=0,ntotal=0;
	int i,ilat,ilatmin,ilatpt,ilon,ilonmin,ilonpt,itim,hrx,minx;
	int om;
	FILE *outclus,*outind;

	taumax0=10.;
	taumin=1.;
	Q=10.;
	P=.90;		/* defaults */
	sd=30.;
	maxdep=30.;
	pw=1./3.;
	pi=3.1415927;
	rad=pi/180.;
	tauterm= -log(1-P);
	fbars=sd*1e6;
	startyear=1800;
	om=2;
	a= -2.31;			/* "Generic Utah Model" */
	b= .87;
	c= .02;
	p= .75;

	outclus=fopen("out.clus","w");
	outind=fopen("out.ind","w");

/* get command line arguments */

	while (--argc) {
	if(**++argv == '-') {
		switch (*(*argv+1)) {
	case 'l':	strcpy(magtyp,"l");break;
	case 'b':strcpy(magtyp,"b");break;
	case 'c':cflag=0;break;
	case 's':strcpy(schar,*++argv);fbars=atof(schar)*1e6;argc--;break;
	case 'h':strcpy(schar,*++argv);maxdep=atof(schar);argc--;break;
	case 'k':strcpy(schar,*++argv);Q=atof(schar);argc--;break;
	case 'p':strcpy(schar,*++argv);P=atof(schar);argc--;break;
	case 'z':strcpy(schar,*++argv);zmin=atof(schar);argc--;break;
	case 'o':strcpy(schar,*++argv);om=atoi(schar);argc--;break;
	case 'q':
		puts("USAGE:  declust [-s bars -h maxdepth -l|w|b ML, Mw, or mb]");
		puts("                [-c 1800s? -k Q factor -p problevel -z mindist]");
		puts("                [-o 1=Calif. 2=Utah]");
		exit(0);
		}}}

c1[2]='\0';c2[2]='\0';c3[2]='\0';c4[2]='\0';c5[2]='\0';
c6[2]='\0';c7[2]='\0';c8[2]='\0';c9[3]='\0';c10[2]='\0';c11[2]='\0';

if(om == 1){
	a= -1.67;		/* "Generic California Model" */
	b= .91;
	c= .05;
	p= 1.08;
	}

/* initialize: read first event */

	fgets(line1,sizeof line1,stdin);
	sscanf(line1,"%*2c%2c%2c%2c%2c%2c%*7c%2c%*1c%2c%2c\
%*1c%3c%*1c%2c%2c%*8c%f",\
c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,&mag1);

year1=atoi(c1);mon=atoi(c2);day=atof(c3);hr=atof(c4);
min=atof(c5);latdeg=atof(c6);latmin=atof(c7);latpt=atof(c8);
londeg=atof(c9);lonmin=atof(c10);lonpt=atof(c11);

/* printf("year1= %d mon= %d day= %4.1f hr= %4.1f min= %4.1f latdeg= %4.1f\
 latmin= %4.1f latpt= %4.1f\n\
londeg= %4.1f lonmin= %4.1f lonpt= %4.1f mag1= %3.1f\n\n",\
year1,mon,day,hr,min,latdeg,latmin,latpt,londeg,lonmin,lonpt,mag1); */

/* get absolute time in days since 1800 */

	if(cflag == 1)yr1=year1+1900; else yr1=year1+1800;
	maintime=(yr1-startyear)*365+cudays[mon-1]+day+hr/24+min/1440;
	time1=maintime;

/* add a day for each leap year since 1800 (1900 is not a leap year)
	and subtract a day if current year is a leap year but month
	is January */

	lpdays=0;
	if(yr1<1901)lpdays=(yr1-startyear-1)/4;
	if(yr1>1904)lpdays=(yr1-startyear-1)/4-1;
	if(yr1>1900 && yr1<1905)lpdays=(yr1-startyear)/4-1;
	maintime += lpdays;
	if((yr1 % 4 == 0 && yr1 % 100 != 0) || yr1 % 400 == 0){
		lpdays++;
		if(mon>2)maintime += 1;
		}

	if(strncmp(magtyp,"l",1) == 0)
		mainmag=.0591*mag1*mag1+.436*mag1+1.13;
	else if(strncmp(magtyp,"b",1) == 0)
		mainmag=2.715-.277*mag1+.127*mag1*mag1;
	else mainmag=mag1;

/* get distance */

	xp=1.5*mainmag+16.1;
	mo=pow(10.,xp);
	t1=.4375*mo/fbars;
	maindist=(pow(t1,pw))/1e5;
	maindist= (maindist>maxdep) ? maxdep : maindist;
	mainlat=latdeg + latmin/60 + latpt/6000;
	mainlon=londeg + lonmin/60 + lonpt/6000;

	ntotal++;

/* end initialization */

	while(fgets(line2,sizeof line2,stdin) != NULL){
	sscanf(line2,"%*2c%2c%2c%2c%2c%2c%*7c%2c%*1c%2c%2c\
%*1c%3c%*1c%2c%2c%*8c%f",\
c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,&mag2);

year2=atoi(c1);mon=atoi(c2);day=atof(c3);hr=atof(c4);
min=atof(c5);latdeg=atof(c6);latmin=atof(c7);latpt=atof(c8);
londeg=atof(c9);lonmin=atof(c10);lonpt=atof(c11);

	ntotal++;

/* printf("year2= %d mon= %d day= %4.1f hr= %4.1f min= %4.1f latdeg= %4.1f\
 latmin= %4.1f latpt= %4.1f\n\
londeg= %4.1f lonmin= %4.1f lonpt= %4.1f mag2= %3.1f\n\n",\
year2,mon,day,hr,min,latdeg,latmin,latpt,londeg,lonmin,lonpt,mag2); */

/* get absolute time in days since startyear (1800) */


	if(cflag == 0 && (year2 < year1))cflag=1;
	if(cflag == 1)yr2=year2+1900; else yr2=year2+1800;
	time2=(yr2-startyear)*365+cudays[mon-1]+day+hr/24+min/1440;

/* add a day for each leap year since 1800 (1900 is not a leap year)
	and subtract a day if current year is a leap year but month
	is January or February */

	lpdays=0;
	if(yr2<1901)lpdays=(yr2-startyear-1)/4;
	if(yr2>1904)lpdays=(yr2-startyear-1)/4-1;
	if(yr2>1900 && yr2<1905)lpdays=(yr2-startyear)/4-1;
	time2 += lpdays;
	if((yr2 % 4 == 0 && yr2 % 100 != 0) || yr2 % 400 == 0){
		lpdays++;
		if(mon>2)time2 += 1;
		}

/* estimate minmag and location error for this year 
	(arrived at by R. LaForge, from guesses and various references 
	for western Montana */

	if(yrmon>=186901 && yrmon <190001){minmag=6.5;zmin=100.;}
	if(yrmon>=190001 && yrmon <192501){minmag=6.0;zmin=100.;}
	if(yrmon>=192501 && yrmon <193501){minmag=5.5;zmin=70.;}
	if(yrmon>=193501 && yrmon <194801){minmag=5.0;zmin=50.;}
	if(yrmon>=194801 && yrmon <195901){minmag=4.5;zmin=40.;}
	if(yrmon>=195901 && yrmon <196301){minmag=4.0;zmin=30.;}
	if(yrmon>=196301 && yrmon <198201){minmag=3.5;zmin=25.;}
	if(yrmon>=198201 && yrmon <199601){minmag=3.0;zmin=20.;}
	taumax=taumax0*pow(2,minmag-2);
	yrmon=yr2*100+mon;
	
	
/* do magnitude conversion if necessary */

	if(strncmp(magtyp,"l",1) == 0)
		mwf2=.0591*mag2*mag2+.436*mag2+1.13;
	else if(strncmp(magtyp,"b",1) == 0)
		mwf2=2.715-.277*mag2+.127*mag2*mag2;
	else mwf2=mag2;

/*  get distances */

	xp=1.5*mwf2+16.1;
	mo=pow(10.,xp);
	t1=.4375*mo/fbars;
	r2=Q*pow(t1,pw)/1e5;
	r2= (r2>maxdep) ? maxdep : r2;
	
	lat2=latdeg + latmin/60 + latpt/6000;
	lon2=londeg + lonmin/60 + lonpt/6000;
 printf("\nevent pair %d\n",ntotal-1); 
	cosavlat=cos((.5*(mainlat+lat2))*rad);
	dlat=fabs(lat2-mainlat)*km;
	dlon=fabs(lon2-mainlon)*cosavlat*km;
	ddist=sqrt(dlat*dlat+dlon*dlon);

/* get time window for event 2 */

	delt=time2-maintime;
	delt2=time2-time1;
/* printf("time1= %f time2= %f maintime= %f\n",time1,time2,maintime); */
	delm=mainmag-minmag-1; 
	if(om == 0)tau=tauterm*delt/pow(10,.6667*delm);
	else {
		term1= pow(10,a+b*(mainmag-mwf2));
		term2= pow(delt+c,p);
		tau=tauterm*term2/term1;
		}
	tau=(tau<taumin) ? taumin : tau; 
	tau=(tau>taumax) ? taumax : tau; 
 printf("delt2= %f tau= %6.3f\n",delt2,tau); 
	maindist=(maindist>maxdep) ? maxdep : maindist;
	zdist=maindist+r2;
	zdist=(zdist<zmin) ? zmin : zdist; 
 printf("ddist= %f zdist= %f\n",\
ddist,zdist); 
printf("mainmag= %3.1f mwf2= %3.1f\n",mainmag,mwf2); 
printf("yr1= %d yr2= %d\n",yr1,yr2); 

/* process event */

	if(ddist < zdist && delt2 < tau){ 		/* linked */
		if(cswitch == 0){          		  /* new cluster */
		nclusters++;
		fprintf(outclus,"Cluster %d\n",nclusters);
		fprintf(outclus,"%s",line1);
		fprintf(outclus,"%s",line2);
		cluslat[nclus]=mainlat;
		cluslon[nclus]=mainlon;
		clusmag[nclus]=mainmag;
		clustim[nclus]=maintime;
		nclus++;
		cluslat[nclus]=lat2;
		cluslon[nclus]=lon2;
		clusmag[nclus]=mwf2;
		clustim[nclus]=time2;
		cswitch=1;
		}else{
		fprintf(outclus,"%s",line2);
		nclus++;
		cluslat[nclus]=lat2;
		cluslon[nclus]=lon2;
		clusmag[nclus]=mwf2;
		clustim[nclus]=time2;
		}
		if(mwf2>mainmag){
			mainlat=lat2;
			mainlon=lon2;
			mainmag=mwf2;
			maintime=time2;
			maindist=r2/Q;
			}
	time1=time2;
	}else{   /* not linked */
		mainlat=lat2;
		mainlon=lon2;
		mainmag=mwf2;
		maintime=time2;
		time1=time2;
		maindist=r2/Q;
		if(cswitch == 0){
			fprintf(outind,"%s",line1);
			nind++;
		}else{     /* last event of cluster */
		cswitch=0;
		sumlat=sumlon=sumwt=sumtim=0.;
		for(i=0;i<=nclus;i++){		/* get equivalent event */
			xp=1.5*clusmag[i]+16.1;
			wt=pow(10,xp);
			sumwt += wt;
			sumlat += wt*cluslat[i];
			sumlon += wt*cluslon[i];
			sumtim += wt*clustim[i];
			}
		avlat=sumlat/sumwt;
		avlon=sumlon/sumwt;
		avtim=sumtim/sumwt;
		abdays();
		itim=avtim;
		fppart=(avtim-itim)*24;
		hrx=fppart;
		fppart -= hrx;
		minx=fppart*60;
		newmag=.667*log10(sumwt)-10.7;
		ilat=avlat;
		fppart=(avlat-ilat)*60;
		ilatmin=fppart;
		fppart -= ilatmin;
		ilatpt=fppart*100;
		ilon=avlon;
		fppart=(avlon-ilon)*60;
		ilonmin=fppart;
		fppart -= ilonmin;
		ilonpt=fppart*100;
		fprintf(outind,"AE%2d%2d%2d%2d%2d       %2dN%2d%2d %3dW%2d%\
2d        %3.1f\n",yrx,monx,dayx,hrx,minx,ilat,ilatmin,ilatpt,ilon,\
			ilonmin,ilonpt,newmag);
		nclus=0; nind++;	
		}							
	}
	strcpy(line1,line2);
	yr1=yr2;
	time1=time2;
	}				/* end event 2 read */
	
	/* process last point */	

	if(ddist > zdist || delt2 > tau)   /* not linked */
	fprintf(outind,"%s\n",line2);
	else{
		if(cswitch == 0){
			nclusters++;
			fprintf(outclus,"Cluster %d\n",nclusters);
			fprintf(outclus,"%s",line1);
			nclus++; 
			cluslat[nclus]=mainlat;
			cluslon[nclus]=mainlon;
			clusmag[nclus]=mainmag;
			clustim[nclus]=maintime; 
			nclus++; 
			cluslat[nclus]=lat2;
			cluslon[nclus]=lon2;
			clusmag[nclus]=mwf2;
			clustim[nclus]=time2; 
			}

		sumlat=sumlon=sumwt=sumtim=0.;
		for(i=0;i<=nclus;i++){		/* get equivalent event */
			xp=1.5*clusmag[i]+16.1;
			wt=pow(10.,xp);
			sumwt += wt;
			sumlat += wt*cluslat[i];
			sumlon += wt*cluslon[i];
			sumtim += wt*clustim[i];
			}
		avlat=sumlat/sumwt;
		avlon=sumlon/sumwt;
		avtim=sumtim/sumwt;
		abdays();
		itim=avtim;
		fppart=(avtim-itim)*24;
		hrx=fppart;
		fppart -= hrx;
		minx=fppart*60;
		newmag=.667*log10(sumwt)-10.7;
		ilat=avlat;
		fppart=(avlat-ilat)*60;
		ilatmin=fppart;
		fppart -= ilatmin;
		ilatpt=fppart*100;
		ilon=avlon;
		fppart=(avlon-ilon)*60;
		ilonmin=fppart;
		fppart -= ilonmin;
		ilonpt=fppart*100;
		fprintf(outind,"AE%2d%2d%2d%2d%2d       %2dN%2d%2d %3dW%2d%\
2d        %3.1f\n",yrx,monx,dayx,hrx,minx,ilat,ilatmin,ilatpt,ilon,\
			ilonmin,ilonpt,newmag);
		nind++;
		}
	
/* print totals */

	printf("no. of independent events= %d\n",nind);		
	printf("no. of clusters= %d\n\n",nclusters);		
	printf("Total no. of events processed= %d\n\n",ntotal);					
}

void abdays(){
	int i,inyear,lpdays2,invertime,remain,lp;

	lpdays2=0;
	inyear=avtim/365-1;
	invertime=inyear*365;
	yrx=inyear+startyear;

	if(yrx<1901)lpdays2=(yrx-startyear-1)/4;
	if(yrx>1904)lpdays2=(yrx-startyear-1)/4-1;
	if(yrx>1900 && yrx<1905)lpdays2=(yrx-startyear)/4-1;
	invertime += lpdays2;
	while(1){  	/* see if year is correct */
	if((yrx % 4 == 0 && yrx % 100 != 0) || yrx % 400 == 0){
	lp=1;
	invertime += 366;
	if(invertime>=avtim){
		invertime -= 366;break;
		}}
	else{ invertime += 365;
	if(invertime>=avtim){
		lp=0;
		invertime -= 365;
		break;
		}}
	yrx++;
	}

	remain=avtim-invertime;

	for(i=1;i<13;i++){  /* find correct month */
	if(lp == 0){
	if(remain > cudays[i-1] && remain <= cudays[i]){
		monx=i;
		remain -= cudays[i-1]; break;
		}
	}else{
	if(remain > culdays[i-1] && remain <= culdays[i]){
		monx=i; remain -= culdays[i-1]; break;
	}}}

	dayx=remain;
	yrx -= startyear;
	if(yrx >= 100)yrx -= 100;

	/*printf(" yr= %d mon= %d day= %d\n",yrx,monx,dayx); */
}
